//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2018 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package importer

import (
	"io"
	"os"
	"path/filepath"
)

func (job *Job) normalize(loudness interface{}) error {
	job.Progress.set(StepNormalizing, 0)

	conv, err := job.newNormalizeConverter(loudness)
	if err != nil {
		job.im.errLog.Printf("normalize(): creating normalize converter failed: %v", err)
		return err
	}
	// from here on conv.Close() and conv.Wait() has to be called in any case to
	// reap potential child process zombies

	done := make(chan error)
	go func() {
		src, err := os.OpenFile(filepath.Join(job.workDir, "source"), os.O_RDONLY, 0400)
		if err != nil {
			done <- err
			return
		}

		srcStat, err := src.Stat()
		if err != nil || srcStat.Size() < 0 {
			done <- err
			return
		}

		written, err := io.Copy(&progressWriter{job, StepNormalizing, 0, uint64(srcStat.Size()), conv}, src)
		if err != nil {
			done <- err
			return
		}
		job.im.dbgLog.Printf("normalize(): done copying %d bytes from source", written)
		close(done)
	}()

	select {
	case <-job.ctx.Done():
		conv.Close()
		go conv.Wait() // do the zombie reaping in seperate go routine since we are not interested in the result anyway
		return job.ctx.Err()
	case err = <-done:
	}

	conv.Close()
	convLog, convErr := conv.Wait()
	job.im.dbgLog.Printf("normalize(): converter returned: %v", err)
	if err == nil {
		err = convErr
	}

	for _, l := range convLog {
		job.im.dbgLog.Println(l.Line)
	}
	job.Progress.set(StepNormalizing, 1)
	return err
}
